package com.yutou.mobilecloud.utils;

import com.alibaba.fastjson.JSONArray;
import org.springframework.core.io.FileSystemResource;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.security.MessageDigest;
import java.text.SimpleDateFormat;
import java.util.Base64;
import java.util.Date;
import java.util.Random;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class Tools {
    private static float UPSPower=-1.0f;
    /**
     * 设置Cookie
     *
     * @param response
     * @param key
     * @param value
     * @param time
     */
    public static void setCookie(HttpServletResponse response, String key, String value, int time) {
        Cookie cookie = new Cookie(key, value);
        if (time != -1) {
            cookie.setMaxAge(time);
        }
        cookie.setPath("/");
        response.addCookie(cookie);

    }

    /**
     * 获取Cookie
     *
     * @param request
     * @param key
     * @return
     */
    public static Cookie getCookie(HttpServletRequest request, String key) {
        Cookie[] cookies = request.getCookies();
        try {
            for (Cookie cookie : cookies) {
                if (key != null && cookie.getName().equals(key)) {
                    return cookie;
                }
            }
        } catch (Exception ignored) {

        }

        return null;
    }

    /**
     * 删除Cookie
     *
     * @param request
     * @param response
     * @param key
     * @return
     */
    public static String deleteCookie(HttpServletRequest request, HttpServletResponse response, String key) {
        for (Cookie cookie : request.getCookies()) {
            if (cookie.getName().equals(key)) {
                cookie.setMaxAge(0);
                cookie.setPath("/");
                cookie.setValue(null);
                response.addCookie(cookie);
            }
        }
        return "ok";
    }


    /**
     * 获取项目路径
     *
     * @param request
     * @return
     */
    public static String getPath(HttpServletRequest request) {
        return request.getServletContext().getRealPath("/") + "/";
    }

    /**
     * 获取客户端IP
     *
     * @param request
     * @return
     */
    public static String getRemoteAddress(HttpServletRequest request) {
        String ip = request.getHeader("x-forwarded-for");
        if (ip == null || ip.length() == 0 || ip.equalsIgnoreCase("unknown")) {
            ip = request.getHeader("Proxy-Client-IP");
        }
        if (ip == null || ip.length() == 0 || ip.equalsIgnoreCase("unknown")) {
            ip = request.getHeader("WL-Proxy-Client-IP");
        }
        if (ip == null || ip.length() == 0 || ip.equalsIgnoreCase("unknown")) {
            ip = request.getRemoteAddr();
        }
        return ip;
    }

    /**
     * N以内的不重复随机数
     *
     * @param min 最小值
     * @param max 最大值
     * @param n
     * @return
     */
    public static int[] randomCommon(int min, int max, int n) {
        int len = max - min + 1;
        if (max < min || n > len) {
            return new int[0];
        }
        // 初始化给定范围的待选数组
        int[] source = new int[len];
        for (int i = min; i < min + len; i++) {
            source[i - min] = i;
        }
        int[] result = new int[n];
        Random rd = new Random();
        int index = 0;
        for (int i = 0; i < result.length; i++) {
            // 待选数组0到(len-2)随机一个下标
            index = Math.abs(rd.nextInt() % len--);
            // 将随机到的数放入结果集
            result[i] = source[index];
            // 将待选数组中被随机到的数，用待选数组(len-1)下标对应的数替换
            source[index] = source[len];
        }
        return result;
    }


    /**
     * 保存上传的文件
     *
     * @param path 路径
     * @param file 文件
     * @return
     * @throws Exception
     */
    public static String createFile(String path, MultipartFile file, String newFileName) throws Exception {
        String savePath = new File("").getAbsolutePath() + File.separator + "html" + File.separator + path;
        File servicePath = new File(savePath);
        if (!servicePath.exists()) {
            servicePath.mkdirs();
        }
        String fileName = file.getOriginalFilename();
        if (newFileName != null) {
            fileName = newFileName;
        }
        File saveFile = new File(savePath + "/" + fileName);
        if (saveFile.exists()) {
            if (!saveFile.delete()) {
                saveFile = new File(savePath + "/" + fileName.replace(".", "_" + new Date().getTime() + "."));
            }
        }
        file.transferTo(saveFile);
        fileName = URLEncoder.encode(fileName, "UTF-8");
        return path + fileName;
    }


    /**
     * 构造给前端的文件
     *
     * @param file 文件路径
     * @return 前端获取的文件
     */
    public static ResponseEntity<FileSystemResource> getFile(File file) {
        HttpHeaders headers = new HttpHeaders();
        headers.add("Cache-Control", "no-cache, no-store, must-revalidate");
        try {
            headers.add("Content-Disposition", "attachment; filename=" + URLEncoder.encode(file.getName(), "UTF-8"));
        } catch (UnsupportedEncodingException e) {
            headers.add("Content-Disposition", "attachment; filename=" + file.getName());
        }
        headers.add("Pragma", "no-cache");
        headers.add("Expires", "0");
        headers.add("Last-Modified", new Date().toString());
        headers.add("ETag", String.valueOf(System.currentTimeMillis()));
        return ResponseEntity.ok().headers(headers).contentLength(file.length()).contentType(MediaType.parseMediaType("application/octet-stream")).body(new FileSystemResource(file));
    }
    public static String getMd5(String str){
        return bytesToHexString(str.getBytes(StandardCharsets.UTF_8));
    }
    public static String getFileMD5(File file){
        return bytesToHexString(getFileMD5Byte(file));
    }
    public static byte[] getFileMD5Byte(File file) {
        if (!file.isFile()) {
            return null;
        }
        MessageDigest digest = null;
        FileInputStream in = null;
        byte[] buffer = new byte[1024];
        int len;
        try {
            digest = MessageDigest.getInstance("MD5");
            in = new FileInputStream(file);
            while ((len = in.read(buffer, 0, 1024)) != -1) {
                digest.update(buffer, 0, len);
            }
            in.close();
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
        return digest.digest();
    }

    private static String bytesToHexString(byte[] src) {
        StringBuilder stringBuilder = new StringBuilder("");
        if (src == null || src.length <= 0) {
            return null;
        }
        for (byte aSrc : src) {
            int v = aSrc & 0xFF;
            String hv = Integer.toHexString(v);
            if (hv.length() < 2) {
                stringBuilder.append(0);
            }
            stringBuilder.append(hv);
        }
        return stringBuilder.toString();
    }

    public static String base64ToString(String base) {
        base = base.replace(" ", "+");
        try {
            base = new String(Base64.getDecoder().decode(base.replace("\r\n", "").getBytes()));
        } catch (Exception e) {
            try {
                base = URLDecoder.decode(base, StandardCharsets.UTF_8);
                base = base.replace(" ", "+");
                base = new String(Base64.getDecoder().decode(base.replace("\r\n", "").getBytes()));
            } catch (Exception e1) {
                e1.printStackTrace();
            }
        }
        return base;
    }
    public static String stringToBase64(String string){
        return Base64.getEncoder().encodeToString(string.getBytes(StandardCharsets.UTF_8)).replace("+"," ");
    }

    /**
     * 异常输出
     *
     * @param e 异常
     * @return
     */
    public static String getExceptionString(Exception e) {
        StringWriter writer = new StringWriter();
        PrintWriter printWriter = new PrintWriter(writer);
        e.printStackTrace(printWriter);
        printWriter.close();
        return writer.toString();
    }

    public static String getToDayTime() {
        return new SimpleDateFormat("yyyy-MM-dd").format(new Date());
    }

    public static String getToDayNowTimeToFileName() {
        return new SimpleDateFormat("yyyy-MM-dd__HH_mm_ss").format(new Date());
    }


}
